require 'httmultiparty'

class APIException < Exception
end

class CloudElementsConnector

=begin
    Class that allows connection to the Cloud Elements API.
    All connections are made through the invoke method.
    It utilized HTTMultiParty to make http requests.
=end
    class ElementsConnector

        attr_accessor :api_endpoint

        def initialize(api_endpoint='https://console.cloud-elements.com/elements/api-v1')
            @api_endpoint = api_endpoint # Endpoint to hit the api. Defaults to production
            @headers = Hash.new # Headers for some requests
            @headers['Content-Type'] = 'application-json'
            @headers['User-Agent'] = 'elements-connector-ruby'
        end

        private

        def process_request
            # This will do error handling later
        end

        public

        # This method conducts the http request and send back the response.
        # It received a string containing the http method, a hash of query/body parameters
        # and and optional list of file names for requests that deal with posting files.
        def request(httpMethod, params, files)
            case httpMethod
            when 'get' then response = HTTMultiParty.get(@url, :headers => @headers, :query => params)
            when 'post'
                if files
                    files.each { |f| params[f] = File.new(f) }
                    @headers.delete(['Content-Type'])
                    response = HTTMultiParty.post(@url, :headers => @headers, :query => params)
                    @headers['Content-Type'] = 'application-json'
                else
                    response = HTTMultiParty.post(@url, :headers => @headers, :body => params.to_json)
                end
            when 'put' then response = HTTMultiParty.put(@url, :headers => @headers, :body => params.to_json)
            when 'delete' then response = HTTMultiParty.delete(@url, :headers => @headers, :body => params.to_json)
            end
            
            return response
        end

        # This method allows users to interact with the Elements API.
        # It is essentially a copy of the Java ElementsConnector class. 
        def invoke(httpMethod, providerName, headers, apiMethodName, params=nil, files=nil, providerVersion='1')
            @url = "#{@api_endpoint}/#{providerName}/#{providerVersion}/#{apiMethodName}"
            if headers.class() == Hash
                auth_string = "User #{headers[:user_secret]}, Organization #{headers[:organization_secret]}"
            else
                auth_string = "Element #{headers}"
            end
            @headers['Authorization'] = auth_string

            return request(httpMethod, params, files)
        end
    end


    def initialize(providerName, elementToken, base_url='https://console.cloud-elements.com/elements/api-v1/')
        apidocs = HTTMultiParty.get(base_url+providerName)
        if apidocs.has_key? 'errorMsg'
            raise APIException.new "Element Hub for #{providerName} does not exist"
        end

        method_data = []
        method_count = 0
        apidocs['apis'].each do |api|
            method_data[method_count] = Hash.new
            
            method_data[method_count][:elementToken] = elementToken
            method_data[method_count][:providerName] = providerName

            method_data[method_count][:apiMethodName] = api['path'].split('/')[-1]
            method_data[method_count][:httpMethod] = api['operations'][0]["httpMethod"].downcase()
            method_count = method_count + 1    
        end

        method_data.each do |data|
            define_singleton_method data[:apiMethodName] do |params=nil, files=nil|
                client = ElementsConnector.new
                return client.invoke(data[:httpMethod], data[:providerName], data[:elementToken], data[:apiMethodName], params, files)
            end
        end

        if self.respond_to? :ping
            response = self.ping
            if not response['success']
                raise APIException.new response['errorMsg']
            end
        end
    end
end
